%%%%%%%%%%%%%%%%%%  REQUIRES WaveLab Toolbox   %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Denoising of the 1D signals of the wavelab toolbox. The denoising is
% performed by using Discrete Wavelet Transform (DWT) and soft thresholding 
% with universal, minimax, and detection thresholds:
% The signals are
% {'HeaviSine' 'Bumps' 'Blocks' 'Doppler' 'Ramp' 'Cusp' 'Sing' 'HiSine' 
% 'LoSine' 'LinChirp' 'Piece-Polynomial' 'QuadChirp' 'MishMash' 
% 'WernerSorrows' 'Leopold' 'Piece-Regular' 'Riemann' 'HypChirps' 
% 'sineoneoverx' 'Chirps'}
% This demo returns a table containing the estimation risks for these
% signals as well as a figure with these risks versus the signal index.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%        Risks comparison                %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all

SigName={'HeaviSine' 'Bumps' 'Blocks' 'Doppler' 'Ramp' 'Cusp' 'Sing' 'HiSine' 'LoSine' 'LinChirp' 'Piece-Polynomial' 'QuadChirp' 'MishMash' 'WernerSorrows' 'Leopold' 'Piece-Regular' 'Riemann' 'HypChirps' 'sineoneoverx' 'Chirps'}; 
Lsn=length(SigName);
N = 2^11;
WName='sym8';
rho = 7;    % SNR
j=6;   % Maximum le vel concerned by thresholding, j <= 10.
 %%%%%%% Thresholds %%%%%%%%%%%%%%%
lambdaU=sqrt(2*log(N)); % Unversal
lambdaP=0.5*lambdaU+log(1+sqrt(1-1/(N)^2))/lambdaU; % Detection
lambdaM = 0.3936 + 0.1829*(log(N)/log(2)); % Minimax
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
ITER=1;      % Number of Iteration (Monte Carlo)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rsqX=zeros(1,Lsn); % risks for the detection threshold
rsqY=rsqX; % risks for the universal threshold
rsqZ=rsqX; % risks for the minimax threshold
%%
for m=1:Lsn
    SName=char(SigName(m));
    Sig=MakeSignal(SName,N);
    for IndexIter=1:ITER
        rand('seed',0);
        [xSig,ySig] = NoiseMaker(Sig,rho);
        %%%%%%%%%%%  DWT  %%%%%%%%%%%%%%%%%%%%%%%%%%
        [Cd,Ld]=wavedec(ySig,j,WName);
        len=Ld(1);
        fin=length(Cd);
        Cf=Cd;
        Ce=Cd;
        Csd=Cd;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for k=2:j+1
            Cf(len+1:len+Ld(k))= SoftThresh(Cf(len+1:len+Ld(k)), lambdaP);
            Ce(len+1:len+Ld(k))= SoftThresh(Ce(len+1:len+Ld(k)), lambdaU);
            Cd(len+1:len+Ld(k))= SoftThresh(Cd(len+1:len+Ld(k)), lambdaM);
            len=len+Ld(k);
        end
        X=waverec(Cf,Ld,WName);
        Y=waverec(Ce,Ld,WName);
        Z=waverec(Cd,Ld,WName);
        %%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%
        rsqX(m)=rsqX(m)+(1/N)*((xSig-X)*(xSig-X)');  % Detection
        rsqY(m)=rsqY(m)+(1/N)*((xSig-Y)*(xSig-Y)'); % Universal
        rsqZ(m)=rsqZ(m)+(1/N)*((xSig-Z)*(xSig-Z)'); % Minimax
        %%%%%%%%%%%%%%%%%%%%
    end
    rsqX(m)=rsqX(m)/ITER;
    rsqY(m)=rsqY(m)/ITER;
    rsqZ(m)=rsqZ(m)/ITER;
end
rsq=[rsqX;rsqZ;rsqY]; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Figure: Risk comparison per signal  
figure
plot(rsqX);    %   Detection
hold on
plot(rsqZ,'g')  %   Minimax
hold on
plot(rsqY,'r')   % Universal
hold off
xlabel('Signal')
ylabel('Risk')
legend('Detection', 'Minimax', 'Universal')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Export risks in format TeX-Table
[NbL,NbC] = size(rsq);
rsq_col  = reshape(rsq,numel(rsq),1);
rsq_ceil = round(10^4*rsq_col)/10^4;
rsq_str  = num2str(rsq_ceil);
for k =1:length(rsq_str)
    x = rsq_str(k,:);
    K = findstr(rsq_str(k,:),'.');
    if K == 1
        disp(sprintf('Problem 1'));
    end;
    if K == 3
        x = [x(2) '.' x(4:length(x)) '0'];
    end;
    if K > 3
        disp(sprintf('Problem 2'));
    end;
    rsq_str(k,:) = x;
end;
format short
text1=['\begin{table}[htp]'];
text2=['\centering'];
text3=['\begin{tabular}'];
text4=['{| r || c  c  c  c  |} \hline'];
text5=['\small{Signal}&\small{HeaviSine}&\small{Bumps}&\small{Blocks}&\small{Doppler}\\\hline \hline'];
text6=['$r_{\lambda_d}^S$&' num2str(rsq(1,1),4) ' & ' num2str(rsq(1,2),4) ' & ' num2str(rsq(1,3),4) ' & ' num2str(rsq(1,4),4) '\\\hline'];
text7=['$r_{\lambda_m}^S$&' num2str(rsq(2,1),4) '&' num2str(rsq(2,2),4) '&' num2str(rsq(2,3),4) '&' num2str(rsq(2,4),4) '\\\hline'];
text8=['$r_{\lambda_u}^S$&' num2str(rsq(3,1),4) '&' num2str(rsq(3,2),4) '&' num2str(rsq(3,3),4) '&' num2str(rsq(3,4),4) '\\\hline  \hline \hline'];
text9=['\small{Signal}&\small{Ramp}&\small{Cusp}&\small{Sing}&\small{HiSine}\\\hline \hline'];
text10=['$r_{\lambda_d}^S$&' num2str(rsq(1,5),4) '&' num2str(rsq(1,6),4) '&' num2str(rsq(1,7),4) '&' num2str(rsq(1,8),4) '\\\hline'];
text11=['$r_{\lambda_m}^S$&' num2str(rsq(2,5),4) '&' num2str(rsq(2,6),4) '&' num2str(rsq(2,7),4) '&' num2str(rsq(2,8),4) '\\\hline'];
text12=['$r_{\lambda_u}^S$&' num2str(rsq(3,5),4) '&' num2str(rsq(3,6),4) '&' num2str(rsq(3,7),4) '&' num2str(rsq(3,8),4) '\\\hline  \hline \hline'];
text13=['\small{Signal}&\small{LoSine}&\small{LinChirp}&\small{Piece-Poly}&\small{QuadChirp}\\\hline \hline'];
text14=['$r_{\lambda_d}^S$&' num2str(rsq(1,9),4) '&' num2str(rsq(1,10),4) '&' num2str(rsq(1,11),4) '&' num2str(rsq(1,12),4) '\\\hline'];
text15=['$r_{\lambda_m}^S$&' num2str(rsq(2,9),4) '&' num2str(rsq(2,10),4) '&' num2str(rsq(2,11),4) '&' num2str(rsq(2,12),4) '\\\hline'];
text16=['$r_{\lambda_u}^S$&' num2str(rsq(3,9),4) '&' num2str(rsq(3,10),4) '&' num2str(rsq(3,11),4) '&' num2str(rsq(3,12),4) '\\\hline  \hline \hline'];
text17=['\small{Signal}&\small{MishMash}&\small{Werner}&\small{Leopold}&\small{Piece-Reg}\\\hline \hline'];
text18=['$r_{\lambda_d}^S$&' num2str(rsq(1,13),4) '&' num2str(rsq(1,14),4) '&' num2str(rsq(1,15),4) '&' num2str(rsq(1,16),4) '\\\hline'];
text19=['$r_{\lambda_m}^S$&' num2str(rsq(2,13),4) '&' num2str(rsq(2,14),4) '&' num2str(rsq(2,15),4) '&' num2str(rsq(2,16),4) '\\\hline'];
text20=['$r_{\lambda_u}^S$&' num2str(rsq(3,13),4) '&' num2str(rsq(3,14),4) '&' num2str(rsq(3,15),4) '&' num2str(rsq(3,16),4) '\\\hline  \hline \hline'];
text21=['\small{Signal}&\small{Riemann}&\small{HypChirps}&\small{sineoverx}&\small{Chirps}\\\hline \hline'];
text22=['$r_{\lambda_d}^S$&' num2str(rsq(1,17),4) '&' num2str(rsq(1,18),4) '&' num2str(rsq(1,19),4) '&' num2str(rsq(1,20),4) '\\\hline'];
text23=['$r_{\lambda_m}^S$&' num2str(rsq(2,17),4) '&' num2str(rsq(2,18),4) '&' num2str(rsq(2,19),4) '&' num2str(rsq(2,20),4) '\\\hline'];
text24=['$r_{\lambda_u}^S$&' num2str(rsq(3,17),4) '&' num2str(rsq(3,18),4) '&' num2str(rsq(3,19),4) '&' num2str(rsq(3,20),4) '\\\hline'];
text25=['\end{tabular}'];
text26=['\end{table}'];
clc
TABL=char(text1,text2,text3,text4,text5,text6,text7,text8,text9,text10,text11,text12,text13,text14,text15,text16,text17,text18,text19,text20,text21,text22,text23,text24,text25,text26)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: am.atto@telecom-bretagne.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%